#include "search_class_definition_by_name.h"

SgClassDefinition* risc::tools::find_class_definition(std::string class_name,
  std::string qualified_namespace, SgNode* root_node)
{
  // First we try to fiend the native class definition
  std::string class_with_qualifier = qualified_namespace + class_name;

  Rose_STL_Container<SgNode*> class_definitions =
    NodeQuery::querySubTree(root_node, V_SgClassDefinition);

  for(Rose_STL_Container<SgNode*>::iterator
      iter = class_definitions.begin();
      iter != class_definitions.end(); iter++) {

    SgClassDefinition* current_class = isSgClassDefinition(*iter);

    std::string complete_class_namespace = current_class->get_qualified_name();

    if(!complete_class_namespace.compare(class_with_qualifier )){
      return current_class;
    }
  }


  // Okay, up to this point we have no definition. Perhaps the definition
  // is implicit in a type (like sc_channel -> typedef sc_module sc_channel)

  Rose_STL_Container<SgNode*> type_definitions =
    NodeQuery::querySubTree(root_node, V_SgTypedefDeclaration);

  for(Rose_STL_Container<SgNode*>::iterator
      iter = type_definitions.begin();
      iter != type_definitions.end(); iter++) {

    SgTypedefDeclaration *type_decl = isSgTypedefDeclaration(*iter);

    std::string complete_class_namespace = type_decl->get_qualified_name();

    if(!complete_class_namespace.compare(class_with_qualifier )){

      return
        isSgClassDeclaration(isSgClassDeclaration(type_decl->get_declaration())->
        get_definingDeclaration())->get_definition();
    }
  }

  return NULL;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
